function [TrainingTime, TestingTime, TrainingAccuracy, TestingAccuracy] = ourELM(fea_Train, fea_Test, trainLabel,testLabel,Elm_Type, NumberofHiddenNeurons, C,ActivationFunction, lambda, L)


%%%%%%%%%%% Macro definition
REGRESSION = 0;
CLASSIFIER = 1;

%%%%%%%%%%% Load training dataset
T = trainLabel';
P = fea_Train';

%%%%%%%%%%% Load testing dataset
TV.T = testLabel';
TV.P = fea_Test';

NumberofTrainingData = size(P,2);
NumberofTestingData  = size(TV.P,2);
NumberofInputNeurons = size(P,1);

if Elm_Type ~= REGRESSION
   tempT = -1 * ones(length(unique(T)),size(P,2));
   tempTVT = -1 * ones(length(unique(TV.T)),size(TV.P,2));
   % for training label
   for i = 1:length(T)
       tempT(T(i),i) = 1;
   end
   % for testing label
   for i = 1:length(TV.T)
       tempTVT(TV.T(i),i) = 1;
   end
   T = tempT;
   TV.T = tempTVT;
end
% if Elm_Type ~= REGRESSION
%     %%%%%%%%%%%% Preprocessing the data of classification
%     sorted_target = sort(cat(2,T,TV.T),2);            %   'ascend'
%     label = zeros(1,1);                               %   Find and save in 'label' class label from training and testing data sets
%     label(1,1) = sorted_target(1,1);
%     size(sorted_target)
%     size(label)
%     j = 1;
%     for i = 2:(NumberofTrainingData + NumberofTestingData)
%         if sorted_target(1,i) ~= label(1,j)
%             j = j + 1;
%             label(1,j) = sorted_target(1,i);
%         end
%     end
%     number_class = j;
%     NumberofOutputNeurons = number_class;
%        
%     %%%%%%%%%% Processing the targets of training
%     temp_T = zeros(NumberofOutputNeurons, NumberofTrainingData);
%     for i = 1:NumberofTrainingData
%         for j = 1:number_class
%             if label(1,j) == T(1,i)
%                 break; 
%             end
%         end
%         temp_T(j,i) = 1;
%     end
%     T = temp_T*2-1;
% 
%     %%%%%%%%%% Processing the targets of testing
%     temp_TV_T = zeros(NumberofOutputNeurons, NumberofTestingData);
%     for i = 1:NumberofTestingData
%         for j = 1:number_class
%             if label(1,j) == TV.T(1,i)
%                 break; 
%             end
%         end
%         temp_TV_T(j,i) = 1;
%     end
%     TV.T = temp_TV_T*2-1;
% 
% end                                                 %   end if of Elm_Type

%%%%%%%%%%% Calculate weights & biases
start_time_train = cputime;

%%%%%%%%%%% Random generate input weights InputWeight (w_i) and biases BiasofHiddenNeurons (b_i) of hidden neurons
InputWeight = rand(NumberofHiddenNeurons,NumberofInputNeurons)*2-1;
BiasofHiddenNeurons = rand(NumberofHiddenNeurons,1);
tempH = InputWeight*P;
clear P;                                            %   Release input of training data 
ind = ones(1,NumberofTrainingData);
BiasMatrix = BiasofHiddenNeurons(:,ind);              %   Extend the bias matrix BiasofHiddenNeurons to match the demention of H
tempH = tempH + BiasMatrix;

%%%%%%%%%%% Calculate hidden neuron output matrix H
switch lower(ActivationFunction)
    case {'sig','sigmoid'}
        %%%%%%%% Sigmoid 
        H = 1 ./ (1 + exp(-tempH));
    case {'sin','sine'}
        %%%%%%%% Sine
        H = sin(tempH);    
    case {'hardlim'}
        %%%%%%%% Hard Limit
        H = double(hardlim(tempH));
    case {'tribas'}
        %%%%%%%% Triangular basis function
        H = tribas(tempH);
    case {'radbas'}
        %%%%%%%% Radial basis function
        H = radbas(tempH);
        case {'linear'}
        H = tempH;
        %%%%%%%% More activation functions can be added here                
end
clear tempH;                                        %   Release the temparary array for calculation of hidden neuron output matrix H

% % construct the laplacian matrix
% Dist = EuDist2(H');
% 
% k = 20;
% t = 0.03*size(fea_Train,1);
% W = zeros(size(fea_Train,1));
% for i = 1:size(fea_Train,1)
%     [~,Idx] = sort(Dist(i,:));
%     for j = 1:size(fea_Train,1)
%         if ~isempty(find(j == Idx(1:k), 1)) && trainLabel(i) == trainLabel(j)
%             W(i,j) = exp(-sum( (H(:,i)-H(:,j)).^2)/t )*...
%                 (1+exp(-sum( (H(:,i)-H(:,j)).^2)/t ));
%         elseif isempty(find(j == Idx(1:k), 1)) && trainLabel(i) ~= trainLabel(j)
%             W(i,j) = exp(-sum( (H(:,i)-H(:,j)).^2)/t )*...
%                 (1-exp(-sum( (H(:,i)-H(:,j)).^2)/t ));
%         else
%             W(i,j) = 0;    
%         end    
%     end
% end
% L = diag(sum(W)) - W;

%%%%%%%%%%% Calculate output weights OutputWeight (beta_i)
%OutputWeight = pinv(H') * T';                        % implementation without regularization factor //refer to 2006 Neurocomputing paper
%OutputWeight=inv(eye(size(H,1))/C+H * H') * H * T';   % faster method 1 //refer to 2012 IEEE TSMC-B paper
%implementation; one can set regularizaiton factor C properly in classification applications 
%OutputWeight=(eye(size(H,1))/C+H * H') \ H * T';      % faster method 2 //refer to 2012 IEEE TSMC-B paper
%implementation; one can set regularizaiton factor C properly in classification applications

OutputWeight = (H*H' + lambda*H*L*H' + eye(size(H,1))/C) \ H * T';


%If you use faster methods or kernel method, PLEASE CITE in your paper properly: 

%Guang-Bin Huang, Hongming Zhou, Xiaojian Ding, and Rui Zhang, "Extreme Learning Machine for Regression and Multi-Class Classification," submitted to IEEE Transactions on Pattern Analysis and Machine Intelligence, October 2010. 

end_time_train = cputime;
TrainingTime = end_time_train-start_time_train;        %   Calculate CPU time (seconds) spent for training ELM

%%%%%%%%%%% Calculate the training accuracy
Y = (H' * OutputWeight)';                             %   Y: the actual output of the training data
if Elm_Type == REGRESSION
    TrainingAccuracy = sqrt(mse(T - Y));               %   Calculate training accuracy (RMSE) for regression case
end
clear H;

%%%%%%%%%%% Calculate the output of testing input
start_time_test=cputime;
tempH_test=InputWeight*TV.P;
clear TV.P;             %   Release input of testing data             
ind=ones(1,NumberofTestingData);
BiasMatrix=BiasofHiddenNeurons(:,ind);              %   Extend the bias matrix BiasofHiddenNeurons to match the demention of H
tempH_test=tempH_test + BiasMatrix;
switch lower(ActivationFunction)
    case {'sig','sigmoid'}
        %%%%%%%% Sigmoid 
        H_test = 1 ./ (1 + exp(-tempH_test));
    case {'sin','sine'}
        %%%%%%%% Sine
        H_test = sin(tempH_test);        
    case {'hardlim'}
        %%%%%%%% Hard Limit
        H_test = hardlim(tempH_test);        
    case {'tribas'}
        %%%%%%%% Triangular basis function
        H_test = tribas(tempH_test);        
    case {'radbas'}
        
        %%%%%%%% Radial basis function
        H_test = radbas(tempH_test);  
        case {'linear'}
        H_test = tempH_test;
        %%%%%%%% More activation functions can be added here        
end
TY = (H_test' * OutputWeight)';                       %   TY: the actual output of the testing data
end_time_test = cputime;
TestingTime = end_time_test - start_time_test;           %   Calculate CPU time (seconds) spent by ELM predicting the whole testing data

if Elm_Type == REGRESSION
    TestingAccuracy = sqrt(mse(TV.T - TY));            %   Calculate testing accuracy (RMSE) for regression case
end

if Elm_Type == CLASSIFIER
%%%%%%%%%% Calculate training & testing classification accuracy
    MissClassificationRate_Training=0;
    MissClassificationRate_Testing=0;

    for i = 1 : size(T, 2)
        [x, label_index_expected] = max(T(:,i));
        [x, label_index_actual]   = max(Y(:,i));
        if label_index_actual ~= label_index_expected
            MissClassificationRate_Training = MissClassificationRate_Training+1;
        end
    end
    TrainingAccuracy = 1 - MissClassificationRate_Training/size(T,2);
    for i = 1 : size(TV.T, 2)
        [x, label_index_expected] = max(TV.T(:,i));
        [x, label_index_actual]   = max(TY(:,i));
        if label_index_actual ~= label_index_expected
            MissClassificationRate_Testing = MissClassificationRate_Testing+1;
        end
    end
    TestingAccuracy = 1 - MissClassificationRate_Testing/size(TV.T,2);  
end